/*==============================================================================
 $Id: align.c,v 1.2 2005/04/20 14:48:00 jkleinj Exp $ 
 align.c : Local and global alignment routines
 Copyright (C) 2004 John Romein
 see README file for Syntax, Copyright and License
===============================================================================*/

#include "aliss.h"

#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define local			1

static int	    best_x, best_y;
static float	    best_score;
/*static const float  gap_open = GAPO, gap_extend = GAPE;*/


__inline__ static float max(float a, float b)
{
    return a > b ? a : b;
}


static void forward_computation_local(float **matrix, float **dpmat, Seq *seq0, Seq *seq1, float gap_open, float gap_extend)
{
    const int	   h = seq0->length, w = seq1->length;
    int		   x, y;
    float   	   diag, max_x[w], max_y;

    best_score = -1;

    for (x = 0; x < w; x ++)
	{
		matrix[0][x] = max(dpmat[0][x], 0);
		max_x[x]     = 0;

		if (best_score < matrix[0][x])
			best_score = matrix[0][x], best_x = x, best_y = 0;
    }

    for (y = 1; y < h; y ++)
	{
		max_y	     = 0;
		matrix[y][0] = max(dpmat[y][0], 0);

		if (best_score < matrix[y][0])
			best_score = matrix[y][0], best_x = x, best_y = y;

		for (x = 1; x < w; x ++)
		{
			diag	 = matrix[y - 1][x - 1];
			matrix[y][x] = max(max(max(max_y, diag), max_x[x]) + dpmat[y][x], 0);
			diag	-= gap_open;
			max_y	 = max(diag, max_y) - gap_extend;
			max_x[x]	 = max(diag, max_x[x]) - gap_extend;

			if (best_score < matrix[y][x])
				best_score = matrix[y][x], best_x = x, best_y = y;
		}
    }

   /*print_dpmat(matrix, h, w);*/
}


static void forward_computation_global(float **matrix, float **dpmat, Seq *seq0, Seq *seq1, float gap_open, float gap_extend)
{
    const int	h = seq0->length, w = seq1->length;
    int			x, y;
    float   	diag, max_x[w], max_y;

    for (x = 0; x < w; x ++)
	{
		matrix[0][x] = dpmat[0][x];
		max_x[x]     = -99999;
    }

    for (y = 1; y < h; y ++)
	{
		max_y	     = -99999;
		matrix[y][0] = dpmat[y][0];

		if (best_score < matrix[y][0])
			best_score = matrix[y][0], best_x = x, best_y = y;

		for (x = 1; x < w; x ++)
		{
			diag	 = matrix[y - 1][x - 1];
			matrix[y][x] = max(max(max_y, diag), max_x[x]) + dpmat[y][x];
			diag	-= gap_open;
			max_y	 = max(diag, max_y) - gap_extend;
			max_x[x]	 = max(diag, max_x[x]) - gap_extend;
		}
    }

    best_score = -99999;

    for (x = w; -- x >= 0;)
		if (best_score < matrix[h - 1][x])
			best_score = matrix[h - 1][x], best_x = x, best_y = h - 1;

    for (y = h; -- y >= 0;)
		if (best_score < matrix[y][w - 1])
			best_score = matrix[y][w - 1], best_x = w - 1, best_y = y;

   /*print_dpmat(matrix, h, w);*/
}


static void print_sequences(const char *p0, const char *p1)
{
    int i /*, value */;

    putchar('\n');

    for (i = 0; i < 80 && p0[i] != '\0'; i ++)
	putchar(p0[i]);

    putchar('\n');

#if 0
    for (i = 0; i < 80 && p0[i] != '\0'; i ++)
	putchar(!isupper(p0[i]) || !isupper(p1[i]) ? ' ' : p0[i] == p1[i] ? '|' : (value = exchange_matrix[p0[i] - '@'][p1[i] - '@']) < 0 ? ' ' : value == 0 ? '.' : ':');

    putchar('\n');
#endif

    for (i = 0; i < 80 && p1[i] != '\0'; i ++)
	putchar(p1[i]);

    putchar('\n');

    if (i == 80 && p0[80] != '\0')
	{
		putchar('\n');
		print_sequences(p0 + 80, p1 + 80);
    }
}


__inline__ static int approximately_equal(float a, float b)
{
    /* this is what you get if you use floating point ... */

    return a == b || fabsf(a - b) <= .00001F * fabsf(a) + .00001F;
}


static void traceback(float **matrix, float **dpmat, Seq *seq0, Seq *seq1, char *str0, char *str1, int global, float gap_open, float gap_extend)
{
    const int   h = seq0->length, w = seq1->length;
    char	   tmp0[w + h], tmp1[w + h], *p0 = tmp0 + w + h - 1, *p1 = tmp1 + w + h - 1;
    float	   score = best_score;
    int		   x = best_x, y = best_y, gap;

    *p0 = *p1 = '\0';

    if (global)
	{
		if ((gap = w - x - 1) > 0)
			goto skip_x2;

		if ((gap = h - y - 1) > 0)
			goto skip_y2;
    }

    while (x >= 0 && y >= 0)
	{
		* -- p0 = seq0->res[y], * -- p1 = seq1->res[x];
		score = matrix[y][x] - dpmat[y][x];
		-- x, -- y;

		if (!global && approximately_equal(score, 0))
			break;

		if (x < 0)
		{
			gap = y + 1;
			goto skip_y1;
		}

		if (y < 0)
		{
			gap = x + 1;
			goto skip_x1;
		}

		if (!approximately_equal(score, matrix[y][x]))
			for (gap = 1, score += gap_open + gap_extend;; gap ++, score += gap_extend)
			{
				if (x >= gap && approximately_equal(score, matrix[y][x - gap]))
				{
skip_x1:			x -= gap;
skip_x2:			memcpy(p1 -= gap, seq1->res + x + 1, gap);
					memset(p0 -= gap, '.', gap);
					break;
				}

				if (y >= gap && approximately_equal(score, matrix[y - gap][x]))
				{
skip_y1:			y -= gap;
skip_y2:			memcpy(p0 -= gap, seq0->res + y + 1, gap);
					memset(p1 -= gap, '.', gap);
					break;
				}
			}
	}


#ifdef VERBOSE
    print_sequences(p0, p1);
    fprintf(stdout, "\nDP traceback score = %f, termini: seq0 = [%d-%d], seq1 = [%d-%d]\n", 
			best_score, x + 2, best_x + 1, y + 2, best_y + 1);
#endif
    strcpy(str0, p0);
    strcpy(str1, p1);
}


void align(float **dpmat, Seq *seq0, Seq *seq1, char *str0, char *str1, int global, float gapo, float gape)
{
    float **matrix = allocate_matrix(seq0->length, seq1->length);

    (global ? forward_computation_global : forward_computation_local)(matrix, dpmat, seq0, seq1, gapo, gape);
    traceback(matrix, dpmat, seq0, seq1, str0, str1, global, gapo, gape);
    deallocate_matrix(matrix, seq0->length);
}
