/*==============================================================================
 $Id: contactmap.c,v 1.1.1.1 2005/04/15 12:11:53 jkleinj Exp $
 contactmap.c : CAO contact derivation and matching
 Copyright (C) 2003-2005 Kuang Lin, adaptation 2005 by Jens Kleinjung :
	contactmap
 Copyright (C) 2005 Jens Kleinjung : match_contacts
 see README file for Syntax, Copyright and License
===============================================================================*/

#include "aliss.h"

#include <stdio.h>
#include <stdlib.h>
#include "contactmap.h"

/*---------------------------------------------------------------------------*/
void contactmap(Ali *ali, Str *str)
{
	int i, j;
	int nres = 0; /* number of residues */
	int ncb; /* number of Cb atoms */
	float maxcontdist = MAXCONTDIST;
	int anr1 = 0; /* selected atom numbers */
	int anr2 = 0;
	float distance; /* atom distance */
	int allocated = 64;

	/* allocate memory for list of all Cb atoms and contact limits */
	str->contmap.cblist = safe_malloc(str->natom * sizeof(int [2]));
	str->contmap.contlim = safe_malloc((str->natom + 1) * sizeof(int [2]));

	/* generate list of all Cb atoms */
	for (i = 0, ncb = 0; i < str->nallatom; ++ i) 
	{
		/* count residues and initialise contact limits */
		if (strncmp(str->allatom[i].atom_ne, " CA ", 4) == 0)
		{
			str->contmap.contlim[nres][0] = -1;
			str->contmap.contlim[nres][1] = -1;
			++ nres;
		}
		/* record residue number [0] and atom number of all Cb atoms [1] */
		if (strncmp(str->allatom[i].atom_ne, " CB ", 4) == 0)
		{
			str->contmap.cblist[ncb][0] = nres-1;
			str->contmap.cblist[ncb][1] = i;
			++ ncb;
		}
	}

	str->contmap.ncontact = 0;
	/* allocate memory for contact list and contact midpoints */
	str->contmap.contact = safe_malloc(allocated * sizeof(int [2]));
	str->contmap.midpoint = safe_malloc(allocated * sizeof(Vec));

	/* define contacts */
	/* calculate difference vectors between Cbs */
    for (i = 0; i < ncb; ++ i)
	{
		anr1 = str->contmap.cblist[i][1]; /* atom number 1 */
        for (j = i + 1; j < ncb; ++ j) /* local contacts=i+1,2,...; global contacts=i+6,7,... */
		{
			anr2 = str->contmap.cblist[j][1]; /* atom number 2 */

			/* contacts should be <= max. contact distance */	
            if ((distance = atom_atom_distance(str, anr1, anr2)) > maxcontdist)
                continue;

			/* assign residue number (sequence) to contact list */
			str->contmap.contact[str->contmap.ncontact][0] = str->contmap.cblist[i][0];
			str->contmap.contact[str->contmap.ncontact][1] = str->contmap.cblist[j][0];
			/*fprintf (stdout, "%d %d %d\n", str->contmap.cblist[i][0],
				str->contmap.ncontact, str->contmap.cblist[j][0]);*/

			/* assign geometric contact midpoint to contact number */
			contact_midpoint(str, anr1, anr2, &str->contmap.midpoint[str->contmap.ncontact]);

			/* assign contact limits : the first(=[0]) and last(=[1]) contact of each residue */
			/* records the first contact of this residue */
			if (str->contmap.contlim[str->contmap.cblist[i][0]][0] < 0)
				str->contmap.contlim[str->contmap.cblist[i][0]][0] = str->contmap.ncontact;
			/* records the last contact of this residue */
			str->contmap.contlim[str->contmap.cblist[i][0]][1] = str->contmap.ncontact;

			++ str->contmap.ncontact; /* count contacts */

			/* allocate more memory if needed */
			if (str->contmap.ncontact == allocated)
			{
				allocated += 64;
				str->contmap.contact = safe_realloc(str->contmap.contact, allocated * sizeof(int [2]));
				str->contmap.midpoint = safe_realloc(str->contmap.midpoint, allocated * sizeof(Vec));
			}
        }
    }

#ifdef VERBOSE
	fprintf (stdout, "%5d sidechain contacts\n", str->contmap.ncontact);
#endif 
}

/*---------------------------------------------------------------------------*/
/* match contacts */
/* pdb0 is template, pdb1 is scanning peptide */
void match_contacts(int i, int j, int pep, Str *pdb0, Str *pdb1, Ali *ali)
{
	int resinc; /* residue increment */
	int res_i; /* residue i (template) */
	int res_j; /* residue j (scanning peptide) */
	int cn_i; /* contact number i (template) */
	int cn_j; /* contact number j (scanning peptide) */
	int allocated = 64;

	/* allocate memory for list of matched contacts */
	pdb0->contmap.nmatch = 0;
	pdb0->contmap.contmatch = safe_malloc(allocated * sizeof(int [2]));

	/* for the peptide length */
	for (resinc = 0; resinc < pep; ++ resinc)
	{
		res_i = resinc + i; /* current residue i */
		res_j = resinc + j; /* current residue j */

		cn_i = pdb0->contmap.contlim[res_i][0]; /* first contact residue i */
		cn_j = pdb1->contmap.contlim[res_j][0]; /* first contact residue j */

		/* for the query contacts of residue res_j*/
		for ( ; cn_j <= pdb1->contmap.contlim[res_j][1]; ++ cn_j)
		{
			if (cn_j < 0) break; /* break if no contact */

			/* for the template contacts of residue res_i */
			for ( ; cn_i <= pdb0->contmap.contlim[res_i][1]; ++ cn_i)
			{
				if (cn_i < 0) break; /* break if no contact */

				/* match contacts of i and j with smallest distance in sequence space */
				/* progress from short to long distances */
				if ((cn_i + 1 <= pdb0->contmap.contlim[res_i][1]) &&
				   abs((pdb1->contmap.contact[cn_j][1] - pdb1->contmap.contact[cn_j][0]) -
				       (pdb0->contmap.contact[cn_i][1] - pdb0->contmap.contact[cn_i][0])) >
				   abs((pdb1->contmap.contact[cn_j][1] - pdb1->contmap.contact[cn_j][0]) -
				       (pdb0->contmap.contact[cn_i+1][1] - pdb0->contmap.contact[cn_i+1][0])))
				{
					continue; /* contact mismatch, try next template contact */
				}
				else
				{
					/* contact match: record and go to next query contact */
					pdb0->contmap.contmatch[pdb0->contmap.nmatch][0] = cn_i;
					pdb0->contmap.contmatch[pdb0->contmap.nmatch][1] = cn_j;
					if (pdb0->contmap.ncontact == allocated)
					{
						allocated += 64;
						pdb0->contmap.contmatch = safe_realloc(pdb0->contmap.contmatch, allocated * sizeof(int [2]));
					}
					++ cn_i;
					break;
				}

			}
		}
	}
}

/*---------------------------------------------------------------------------*/
/* Euklidean atom-atom distance */
float atom_atom_distance(Str *str, int anr1, int anr2)
{
	return sqrt(pow(str->allatom[anr1].x - str->allatom[anr2].x, 2) + 
				pow(str->allatom[anr1].y - str->allatom[anr2].y, 2) + 
				pow(str->allatom[anr1].z - str->allatom[anr2].z, 2));
}

/*---------------------------------------------------------------------------*/
/* Euklidean atom-vector distance */
float atom_vector_distance(Str *str, int anr1, Vec *vector1)
{
	return sqrt(pow(str->allatom[anr1].x - vector1->x, 2) + 
				pow(str->allatom[anr1].y - vector1->y, 2) + 
				pow(str->allatom[anr1].z - vector1->z, 2));
}

/*---------------------------------------------------------------------------*/
/* Euklidean vector-vector distance */
float vector_vector_distance(Str *str, Vec *vector1, Vec *vector2)
{
	return sqrt(pow(vector1->x - vector2->x, 2) + 
				pow(vector1->y - vector2->y, 2) + 
				pow(vector1->z - vector2->z, 2));
}

/*---------------------------------------------------------------------------*/
/* contact midpoint */
void contact_midpoint(Str *str, int anr1, int anr2, Vec *midpoint)
{
	midpoint->x = (str->allatom[anr1].x + str->allatom[anr2].x) / 2;
	midpoint->y = (str->allatom[anr1].x + str->allatom[anr2].y) / 2;
	midpoint->z = (str->allatom[anr1].z + str->allatom[anr2].z) / 2;
}

