/*==============================================================================
 $Id: newcao.c,v 1.1.1.1 2005/04/15 12:11:53 jkleinj Exp $
 newcao.c : generate CAO matrix
 Copyright (C) 2004 John Romein
 see README file for Syntax, Copyright and License
===============================================================================*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>


#define NR_RESIDUES 20
#define MATRIX_SIZE (NR_RESIDUES * NR_RESIDUES)

typedef float matrix[MATRIX_SIZE][MATRIX_SIZE];
typedef float vector[MATRIX_SIZE];

extern matrix S, S1;
extern vector d, cont_ocu;
extern float  log_res_ocu[NR_RESIDUES];
vector	      L, LV;
matrix	      CAO;


static void matrix_multiply(matrix d, matrix s1, matrix s2)
{
    int i, j, k;

    for (i = 0; i < MATRIX_SIZE; i ++)
	for (j = 0; j < MATRIX_SIZE; j ++) {
	    float sum = 0;

	    for (k = 0; k < MATRIX_SIZE; k ++)
		sum += s1[i][k] * s2[k][j];

	    d[i][j] = sum;
	}
}


void compute_cao(int exp_value)
{
    int    i, j;
    float  t;

    for (i = 0; i < MATRIX_SIZE; i ++) {
	L[i]  = exp(exp_value * d[i]);
	LV[i] = log_res_ocu[i % 20] + log_res_ocu[i / 20];
    }

    for (i = 0; i < MATRIX_SIZE; i ++)
	for (j = 0; j < MATRIX_SIZE; j ++)
	    S[i][j] *= L[j];

    matrix_multiply(CAO, S, S1);

    for (i = 0; i < MATRIX_SIZE; i ++) {
	t = 0;

	for (j = 0; j < MATRIX_SIZE; j ++)
	    t += CAO[i][j];

	t = cont_ocu[i] / t;

	for (j = 0; j < MATRIX_SIZE; j ++)
	    CAO[i][j] *= t;
    }

    for (i = 0; i < MATRIX_SIZE; i ++)
	for (j = i + 1; j < MATRIX_SIZE; j ++)
	    CAO[i][j] = CAO[j][i] = (CAO[i][j] + CAO[j][i]) / 2;

    for (i = 0; i < MATRIX_SIZE; i ++) {
	t = 1 / cont_ocu[i];

	for (j = 0; j < MATRIX_SIZE; j ++)
	    CAO[i][j] = log(CAO[i][j] * t) - LV[j];
    }

#if 0
    for (i = 0; i < MATRIX_SIZE; i ++)
	for (j = 0; j < MATRIX_SIZE; j ++)
	    printf("%f ", CAO[i][j]);
#endif
}


int main(void)
{
    compute_cao(100);
    return 0;
}
